# from django.core.management.base import BaseCommand
# from blogs.models import Popular
# from .analytics_api import print_response
#
#
# class Command(BaseCommand):
#     def handle(self, *args, **options):
#
#         # ブログ閲覧数取得処理
#         Popular.objects.all().delete()
#         for title, path, view in print_response():
#             Popular.objects.create(
#                         title=title, path=path, view=view
#             )
#
#         self.stdout.write(self.style.SUCCESS('更新完了'))



import glob
import os
import re

from django.conf import settings  # type: ignore
from django.core.management.base import BaseCommand  # type: ignore

from blogs.models import Blog


class Command(BaseCommand):
    def handle(self, *args, **options):
        """
        markdownxで蓄積した画像ファイルを整理

        $ python3 manage.py file_cleanup
        """
        img_list = []
        sum_deletes = 0
        blogs = Blog.objects.all()
        for blog in blogs:
            img_datas = re.findall(
                r"(?<=\!\[\]\(/)\w+/\w+/\w+-\w+-\w+-\w+-\w+.\w+", blog.text
            )
            img_datas_2 = re.findall(
                r'(?<=\<img src\="/)\w+/\w+/\w+-\w+-\w+-\w+-\w+.\w+', blog.text
            )
            if img_datas:
                img_list += img_datas
            if img_datas_2:
                img_list += img_datas_2
        if settings.DEBUG:
            candidates_delete = glob.glob("media/markdownx/*.*")
        else:
            img_list = [re.sub(r"^", "/var/www/", img) for img in img_list]
            candidates_delete = glob.glob("/var/www/media/markdownx/*.*")
        for candidate in candidates_delete:
            if candidate not in img_list:
                self.stdout.write(self.style.SUCCESS(f"削除 => {candidate}"))
                os.remove(candidate)
                sum_deletes += 1
        self.stdout.write(self.style.SUCCESS(f"記事中ファイル合計: {len(img_list)}"))
        self.stdout.write(self.style.SUCCESS(f"メディアファイル合計: {len(candidates_delete)}"))
        self.stdout.write(self.style.SUCCESS(f"削除したファイル合計: {sum_deletes}"))
        self.stdout.write(self.style.SUCCESS("クリーンアップ完了"))



# from apiclient.discovery import build
# from oauth2client.service_account import ServiceAccountCredentials
# import os
# import datetime
# from dateutil.relativedelta import relativedelta
# from django.conf import settings
#
#
# SCOPES = ['https://www.googleapis.com/auth/analytics.readonly']
# KEY_FILE_LOCATION = os.path.join(settings.ZEROFROMLIGHT_DIR, 'client_secrets.json')
# VIEW_ID = settings.ZEROFROMLIGHT_KEYS['VIEW_ID']
#
# def initialize_analyticsreporting():
#     credentials = ServiceAccountCredentials.from_json_keyfile_name(
#                             KEY_FILE_LOCATION, SCOPES)
#     analytics = build('analyticsreporting', 'v4', credentials=credentials)
#     return analytics
#
#
# def get_report(analytics):
#     """
#     ブログ閲覧数取得
#
#     """
#     return analytics.reports().batchGet(
#             body={
#             'reportRequests': [{
#                 'viewId': VIEW_ID,
#                 'pageSize': 10,
#                 'dateRanges': [{"startDate": "3daysAgo", "endDate": "today"}],
#                 'dimensions': [{'name': 'ga:pagePath'}, {'name': 'ga:pageTitle'}],
#                 'dimensionFilterClauses': [{'filters': [{'dimensionName': 'ga:pagePath',
#                                                        'expressions': ['/blogs/detail/']}]
#                                            }],
#                 'metrics': [{'expression': 'ga:pageviews'}],
#                 'orderBys': [{'fieldName': 'ga:pageviews', 'sortOrder': 'DESCENDING'}],
#              }]
#             }
#     ).execute()
#
#
# def print_response():
#     """
#     ブログ閲覧数取得処理
#
#     """
#     analytics = initialize_analyticsreporting()
#     response = get_report(analytics)
#     print('結果---------')
#     print(response['reports'][0]['data'])
#
#     for row in response['reports'][0]['data']['rows']:
#         row_path = 'https://zerofromlight.com' + row['dimensions'][0]
#         #row_path = row['dimensions'][0][6:]
#         #row_path = row['dimensions'][0].split('/')[3]
#         row_title = row['dimensions'][1]
#         row_view = row['metrics'][0]['values'][0]
#         yield row_title, row_path, row_view



from django import forms  # type: ignore
from markdownx.widgets import MarkdownxWidget  # type: ignore

from .models import Blog


class BlogForm(forms.ModelForm):
    """
    Blog作成自作フォームの作成

    """

    class Meta:
        model = Blog
        fields = ("category", "title", "text", "is_publick")
        widgets = {
            "category": forms.Select(
                attrs={
                    "class": "form-control",
                    "placeholder": "カテゴリー",
                }
            ),
            "title": forms.TextInput(
                attrs={
                    "class": "form-control",
                    "placeholder": "タイトル",
                }
            ),
            "text": MarkdownxWidget(
                attrs={
                    "class": "form-control",
                    "placeholder": "ブログ内容を入力",
                }
            ),
            "is_publick": forms.RadioSelect,
        }



import io

import japanize_matplotlib  # type: ignore
import matplotlib.pyplot as plt  # type: ignore
import numpy as np  # type: ignore
from django.contrib import messages  # type: ignore
from django.http import HttpResponse  # type: ignore
from django.shortcuts import get_object_or_404, redirect  # type: ignore
from django.urls import reverse_lazy  # type:ignore
from django.views.generic import CreateView, DetailView, ListView, UpdateView, View  # type: ignore
from matplotlib.backends.backend_agg import FigureCanvasAgg  # type: ignore

from .forms import BlogForm
from .models import Blog, Category


class IndexView(ListView):
    template_name = "blogs/index.html"
    context_object_name = "blogs"
    paginate_by = 10

    def get_queryset(self):
        """ブログ記事が公開且つIDの古い順にデータを取得"""
        queryset = Blog.objects.filter(is_publick=True).order_by("-id")
        return queryset


class CategoryView(ListView):
    template_name = "blogs/index.html"
    context_object_name = "blogs"
    paginate_by = 10

    def get_queryset(self):
        """カテゴリー別でブログ記事が公開且つIDの古い順にデータを取得"""
        # 404エラーを使用した方法
        category = get_object_or_404(Category, title=self.kwargs["category"])
        queryset = Blog.objects.filter(is_publick=True, category=category).order_by("-id")
        messages.success(self.request, f"カテゴリ:{category}")
        return queryset


class BlogDetailView(DetailView):
    model = Blog
    template_name = "blogs/detail.html"
    context_object_name = "blog"

    def get_context_data(self, **kwargs):
        """テンプレートへ渡す新着記事のインスタンスの作成"""
        context = super().get_context_data(**kwargs)
        context["new_articls"] = self.model.objects.filter(is_publick=True).order_by(
            "-id"
        )[:5]
        return context


class PrivateIndexView(ListView):
    queryset = Blog.objects.filter(is_publick=False).order_by("-id")
    template_name = "blogs/private_index.html"
    context_object_name = "private_blog"

    def get(self, request):
        """管理人以外のアクセスはHTMLページでコメントを返す"""
        if not request.user.is_authenticated:
            return HttpResponse("<h1>権限がありません。</h1>")
        return super().get(request)


class PrivateDetailView(DetailView):
    model = Blog
    template_name = "blogs/private_detail.html"
    # context_object_name = 'private_detail'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["private_detail"] = get_object_or_404(
            Blog, id=self.kwargs["pk"], is_publick=False
        )
        return context

    def get(self, request, pk):
        """管理人以外のアクセスはHTMLページでコメントを返す"""
        if not request.user.is_authenticated:
            return HttpResponse("<h1>権限がありません。</h1>")
        return super().get(request, pk)


class BlogFormView(CreateView):
    model = Blog
    form_class = BlogForm
    template_name = "blogs/new_blog.html"
    success_url = reverse_lazy("blogs:index")
    get_object_name = "blog"

    def form_valid(self, form):
        """検証が終わったらメッセージを表示する"""
        messages.success(self.request, "新規作成完了")
        return super().form_valid(form)

    def get(self, request):
        """管理人以外のアクセスはHTMLページでコメントを返す"""
        if not request.user.is_authenticated:
            return HttpResponse("<h1>権限がありません。</h1>")
        return super().get(request)


class EditBlogFormView(UpdateView):
    """更新用フォーム"""

    model = Blog
    form_class = BlogForm
    template_name = "blogs/edit_blog.html"
    success_url = reverse_lazy("blogs:index")
    get_object_name = "blog"

    def form_valid(self, form):
        """検証が終わったらメッセージを表示する"""
        messages.success(self.request, "更新完了")
        return super().form_valid(form)

    def get(self, request, pk):
        """管理人以外のアクセスはHTMLページでコメントを返す"""
        if not request.user.is_authenticated:
            return HttpResponse("<h1>権限がありません。</h1>")
        return super().get(request, pk)


def release(request, pk):
    """Blog公開用"""
    if request.user.is_authenticated:
        blog_release = get_object_or_404(Blog, id=pk, is_publick=False)
        blog_release.to_release()
        return redirect("blogs:index")
    return HttpResponse("<h1>権限がありません。</h1>")


def private(request, pk):
    """Blog非公開用"""
    if request.user.is_authenticated:
        blog_private = get_object_or_404(Blog, id=pk, is_publick=True)
        blog_private.to_private()
        return redirect("blogs:private_index")
    return HttpResponse("<h1>権限がありません。</h1>")


class CategoryGraphView(View):
    template_name = "blogs/edit_blog.html"

    def get(self, request):
        """Blogカテゴリーのグラフ"""

        # リンターで引っかかるので定義
        _ = japanize_matplotlib
        plt.rcParams.update({"figure.autolayout": True})
        fig = plt.figure(facecolor="whitesmoke")
        ax = fig.add_subplot(1, 1, 1)
        ax.set_facecolor("whitesmoke")  # プロット内の背景色

        # グラフの枠線を削除
        plt.gca().spines["right"].set_visible(False)
        plt.gca().spines["top"].set_visible(False)
        plt.gca().spines["bottom"].set_visible(False)
        plt.gca().spines["left"].set_visible(False)

        # ラベルの補助線を削除
        plt.tick_params(bottom=False, left=False, right=False, top=False)

        """ここにデータを作成する"""
        category_choice = Category.objects.all()
        blogs_choice = Blog.objects.select_related("category").all()
        x1 = [data.title for data in category_choice]
        y1 = [data.category_id for data in blogs_choice]
        y1 = np.unique(y1, return_counts=True)
        colorlist = ["r", "y", "g", "b", "m", "c", "#ffff33", "#f781bf"]
        ax.bar(x1, y1[1], color=colorlist, width=0.3, alpha=0.5)
        buf = io.BytesIO()
        canvas = FigureCanvasAgg(fig)
        canvas.print_png(buf)
        response = HttpResponse(buf.getvalue(), content_type="image/png")
        fig.clear()
        response["Content-Length"] = str(len(response.content))

        # 引数requestがリンターで引っかかるので無理に定義しているだけ
        request.close()
        return response



from django.urls import path  # type: ignore

from . import views

app_name = "blogs"

urlpatterns = [
    path("", views.IndexView.as_view(), name="index"),
    path("category/<str:category>/", views.CategoryView.as_view(), name="category"),
    path("detail/<int:pk>/", views.BlogDetailView.as_view(), name="detail"),
    path("private_index/", views.PrivateIndexView.as_view(), name="private_index"),
    path(
        "private_index/private_detail/<int:pk>/",
        views.PrivateDetailView.as_view(),
        name="private_detail",
    ),
    path("new_blog/", views.BlogFormView.as_view(), name="new_blog"),
    path("edit_blog/<int:pk>/", views.EditBlogFormView.as_view(), name="edit_blog"),
    path("release/<pk>/", views.release, name="release"),
    path("private/<pk>/", views.private, name="private"),
    path("category_graph/", views.CategoryGraphView.as_view(), name="category_graph"),
]



from django.test import TestCase  # type: ignore

from blogs.templatetags.mark import markdown_to_html

from .models import Blog, Category


class MarkTests(TestCase):
    """blogs/templatetags/mark.py"""

    def test_markdown_to_html(self):
        """マークダウンからHTMLへ変換"""
        text = "*Hello*"
        html = markdown_to_html(text)
        self.assertEqual(html, "<p><em>Hello</em></p>")


class CategoryModelTests(TestCase):
    """Categoryモデルのテスト"""

    def test_category_title_result(self):
        """__str__メソッド"""
        result = Category(title="カテゴリー１")
        self.assertIsInstance(result.__str__(), str)


class BlogModelTests(TestCase):
    """Blogモデルのテスト"""

    def test_blog_to_release_result(self):
        """to_releaseメソッド"""
        category = Category.objects.create(title="カテゴリー１")
        result = category.blog_set.create(title="タイトル", text="テキスト", is_publick=False)
        result.to_release()
        self.assertIs(result.is_publick, True)

    def test_blog_to_private_result(self):
        """to_privateメソッド"""
        category = Category.objects.create(title="カテゴリー１")
        result = category.blog_set.create(title="タイトル", text="テキスト", is_publick=True)
        result.to_private()
        self.assertIs(result.is_publick, False)

    def test_blog_get_toc_result(self):
        """get_tocメソッド"""
        category = Category.objects.create(title="カテゴリー１")
        text = """[TOC]\n## Hello"""
        blog = category.blog_set.create(
            title="タイトル", text="[TOC]\n## Hello", is_publick=True
        )
        text = """<div class="toc">\n<ul>\n<li><a href="#hello">Hello</a></li>\n</ul>\n</div>\n"""
        self.assertEqual(blog.get_toc(), text)

    def test_blog_title_result(self):
        """__str__メソッド"""
        result = Blog(title="タイトル")
        self.assertIsInstance(result.__str__(), str)
